
# q-expansion of newform 2535.2.a.j, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2535
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [-1], [-1], [0], [4], [0], [2], [-4], [0], [-2], [0], [10], [-10], [4], [-8], [-10], [4], [-2], [-12], [8], [-10], [0], [-12], [6], [-2], [6], [-16], [-12], [-14], [2], [-8], [-12], [6], [-4], [-22], [8], [14], [4], [0], [-18], [20], [-10], [16], [-2], [-6], [-8], [20], [-8], [20], [-6], [-6], [16], [14], [12], [18], [16], [14], [-16], [6], [6], [-12], [-6], [-28], [-24], [26], [2], [-12], [-14], [-28], [2], [-18], [24], [-24], [-26], [20], [24], [6], [2], [-18], [-26], [4], [26], [0], [-14], [40], [-12], [-2], [-10], [18], [-24], [28], [0], [-32], [28], [-4], [-32], [34], [10], [4], [-30], [-20], [18], [12], [-6], [-4], [-2], [12], [-34], [-8], [26], [-8], [-22], [6], [4], [8], [-30], [36], [32], [46], [20], [-22], [-30], [6], [-36], [44], [-2], [26], [-48], [-16], [-14], [44], [16], [16], [-26], [6], [-2], [-6], [-28], [-2], [10], [-12], [-54], [32], [28], [30], [-40], [-6], [-22], [-20], [56], [-30], [-46], [44], [48], [-12], [32], [40], [-34], [-54], [50], [36], [-22], [-32], [60], [-2], [0], [32], [54], [-50], [22], [36], [2], [24], [54], [-32], [-6], [-60], [-6], [-16], [-50], [40], [-44], [22], [-58], [-24], [-10], [-18], [-28], [22], [16], [34], [28], [-12], [34], [12], [-38], [14], [-18], [-18], [16], [18], [48], [-6], [34], [-12], [30], [16], [-4], [-42], [44], [-46], [22], [0], [28], [24], [22], [-56], [18], [-16], [66], [-26], [-8], [30], [8], [28], [-10], [-54], [64], [-32], [36], [30], [4], [64], [-22], [-28], [-72], [-18], [74], [-36], [-72], [-12], [-36], [16], [18], [-66], [56], [-56], [-20], [4], [56], [50], [-2], [0], [-54], [46], [12], [22], [28], [6], [-26], [-56], [-20], [-38], [-18], [78], [52], [-14], [-58], [-20], [-58], [14], [-28], [-26], [48], [18], [64], [-36], [-30], [54], [-60], [40], [56], [-80], [-6], [12], [0], [-14], [-54], [-8], [-30], [46], [28], [54], [36], [-34], [30], [32], [-42], [60], [-44], [42], [-14], [-32], [12], [28], [-2], [-12], [14], [-24], [54], [24], [-54], [-30], [-4], [32], [-86], [12], [48], [30], [18], [20], [70], [-34], [56], [6], [-14], [-20], [44], [24], [-58], [18], [62], [80], [-28], [52], [-4], [-62], [-82], [22], [72], [26], [10], [-6], [72], [-62], [-60], [38], [44], [0], [-74], [-36], [54], [50], [88], [54], [42], [-64], [36], [66], [-32], [-22], [74], [-24], [4], [-52], [10], [18], [48], [26], [84], [-36], [40], [54], [24], [-50], [76], [48], [-98], [-78], [58], [98], [6], [-16], [-82], [12], [16], [16], [38], [-84], [-72], [-66], [58], [52], [-100], [-8], [26], [-80], [-54], [-20], [-54], [98], [18], [-40], [-10], [26], [40], [-62], [14], [84], [-36], [18], [-42], [20], [36], [-22], [14], [-64], [-32], [30], [-32], [-98], [74], [24], [-4], [22], [82], [-20], [-90], [4], [56]]
